/* eslint-disable no-magic-numbers */
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { renderHook } from '@testing-library/react';
import { DEFAULT_ERROR } from '@/constants/errors';
import { overlayFixture, overlaysFixture } from '@/models/fixtures/overlaysFixture';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { apiPath } from '@/redux/apiPath';
import { HttpStatusCode } from 'axios';
import { act } from 'react-dom/test-utils';
import { OVERLAYS_INITIAL_STATE_MOCK } from '@/redux/overlays/overlays.mock';
import { useUserOverlays } from './useUserOverlays';

const mockedAxiosClient = mockNetworkResponse();

describe('useUserOverlays', () => {
  it('should not fetch user overlays on mount if login does not exist', async () => {
    const { Wrapper, store } = getReduxStoreWithActionsListener({
      user: {
        authenticated: false,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
        login: null,
        role: 'user',
      },
      overlays: OVERLAYS_INITIAL_STATE_MOCK,
    });

    const {
      result: {
        current: { userOverlaysList },
      },
    } = renderHook(() => useUserOverlays(), {
      wrapper: Wrapper,
    });

    const actions = store.getActions();
    const firstAction = actions[0];

    expect(firstAction).toBeUndefined();
    expect(userOverlaysList).toEqual([]);
  });
  it('should store fetched user overlays to userOverlaysList state', () => {
    mockedAxiosClient
      .onGet(
        apiPath.getAllUserOverlaysByCreatorQuery({
          publicOverlay: false,
          creator: 'test',
        }),
      )
      .reply(HttpStatusCode.Ok, overlaysFixture);

    const { Wrapper } = getReduxStoreWithActionsListener({
      user: {
        authenticated: true,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
        login: 'test',
        role: 'user',
      },
      overlays: {
        ...OVERLAYS_INITIAL_STATE_MOCK,
        userOverlays: {
          data: overlaysFixture,
          loading: 'idle',
          error: DEFAULT_ERROR,
        },
      },
    });

    const {
      result: {
        current: { userOverlaysList },
      },
    } = renderHook(() => useUserOverlays(), {
      wrapper: Wrapper,
    });

    expect(userOverlaysList).toEqual(overlaysFixture);
  });
  it('should move user overlay list item on order change', async () => {
    const FIRST_USER_OVERLAY = overlayFixture;
    const SECOND_USER_OVERLAY = overlayFixture;
    mockedAxiosClient
      .onGet(
        apiPath.getAllUserOverlaysByCreatorQuery({
          publicOverlay: false,
          creator: 'test',
        }),
      )
      .reply(HttpStatusCode.Ok, [FIRST_USER_OVERLAY, SECOND_USER_OVERLAY]);

    const { Wrapper } = getReduxStoreWithActionsListener({
      user: {
        authenticated: true,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
        login: 'test',
        role: 'user',
      },
      overlays: {
        ...OVERLAYS_INITIAL_STATE_MOCK,
        userOverlays: {
          data: [FIRST_USER_OVERLAY, SECOND_USER_OVERLAY],
          loading: 'idle',
          error: DEFAULT_ERROR,
        },
      },
    });

    const {
      result: {
        current: { moveUserOverlayListItem, userOverlaysList },
      },
    } = renderHook(() => useUserOverlays(), {
      wrapper: Wrapper,
    });

    await act(() => {
      moveUserOverlayListItem(0, 1);
    });

    expect(userOverlaysList).toEqual([SECOND_USER_OVERLAY, FIRST_USER_OVERLAY]);
  });
  it('calls updateOverlays on calling updateUserOverlaysOrder', async () => {
    const FIRST_USER_OVERLAY = { ...overlayFixture, order: 1, idObject: 12 };
    const SECOND_USER_OVERLAY = { ...overlayFixture, order: 2, idObject: 92 };
    mockedAxiosClient
      .onGet(
        apiPath.getAllUserOverlaysByCreatorQuery({
          publicOverlay: false,
          creator: 'test',
        }),
      )
      .reply(HttpStatusCode.Ok, [FIRST_USER_OVERLAY, SECOND_USER_OVERLAY]);

    const { Wrapper, store } = getReduxStoreWithActionsListener({
      user: {
        authenticated: true,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
        login: 'test',
        role: 'user',
      },
      overlays: {
        ...OVERLAYS_INITIAL_STATE_MOCK,
        userOverlays: {
          data: [FIRST_USER_OVERLAY, SECOND_USER_OVERLAY],
          loading: 'idle',
          error: DEFAULT_ERROR,
        },
      },
    });

    const {
      result: {
        current: { moveUserOverlayListItem, updateUserOverlaysOrder },
      },
    } = renderHook(() => useUserOverlays(), {
      wrapper: Wrapper,
    });

    await act(() => {
      moveUserOverlayListItem(0, 1);
    });

    updateUserOverlaysOrder();

    const actions = store.getActions();

    const firstAction = actions[0];
    expect(firstAction.type).toBe('overlays/updateOverlays/pending');
  });
});
