/* eslint-disable no-magic-numbers */
import { StoreType } from '@/redux/store';
import { render, screen } from '@testing-library/react';
import { act } from 'react-dom/test-utils';
import { initialMapDataFixture, initialMapStateFixture } from '@/redux/map/map.fixtures';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import {
  BACKGROUNDS_MOCK,
  BACKGROUND_INITIAL_STATE_MOCK,
} from '@/redux/backgrounds/background.mock';
import { BackgroundSelector } from './BackgroundsSelector.component';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <BackgroundSelector />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('BackgroundSelector - component', () => {
  it('should initialy display default value', () => {
    renderComponent();

    const buttonName = screen.getByTestId('background-dropdown-button-name');

    expect(buttonName.textContent).toBe('Background');
  });

  it("should display selected value name when it's not main background", async () => {
    renderComponent({
      map: initialMapStateFixture,
      backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
    });

    const buttonName = screen.getByTestId('background-dropdown-button-name');
    await act(() => {
      buttonName.click();
    });
    expect(buttonName.textContent).toBe('Background');

    const backgroundButton = screen.getByText('Network');
    await act(() => {
      backgroundButton.click();
    });

    expect(buttonName.textContent).toBe('Network');
  });

  it('should change redux map state on selecting background', async () => {
    const { store } = renderComponent({
      map: initialMapStateFixture,
      backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
    });
    expect(store.getState().map.data.modelId).toBe(0);

    const buttonName = screen.getByTestId('background-dropdown-button-name');
    await act(() => {
      buttonName.click();
    });

    const backgroundButton = screen.getByText('Network');
    await act(() => {
      backgroundButton.click();
    });

    expect(store.getState().map.data.backgroundId).toBe(14);
  });

  describe('query params', () => {
    it('should display default value when main background id is in query params', async () => {
      await renderComponent({
        map: { ...initialMapStateFixture, data: { ...initialMapDataFixture, backgroundId: 13 } },
        backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
      });

      const buttonName = screen.getByTestId('background-dropdown-button-name');
      expect(buttonName.textContent).toBe('Background');
    });
    it('should display correct background when background id is in query params', async () => {
      await renderComponent({
        map: { ...initialMapStateFixture, data: { ...initialMapDataFixture, backgroundId: 15 } },
        backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
      });

      const buttonName = screen.getByTestId('background-dropdown-button-name');
      expect(buttonName.textContent).toBe('Empty');
    });
    it.skip('should set backgroundid in query on selecting background', () => {
      // TODO e2e
      expect(true).toBe(false);
    });
  });
});
