/* eslint-disable no-magic-numbers */
import { FIRST_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { reactionsFixture } from '@/models/fixtures/reactionFixture';
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { mockNetworkNewAPIResponse, mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { HttpStatusCode } from 'axios';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import * as findClosestReactionPoint from './findClosestReactionPoint';
import { handleReactionResults } from './handleReactionResults';

const mockedAxiosOldClient = mockNetworkResponse();
const mockedAxiosNewClient = mockNetworkNewAPIResponse();

jest.mock('../../../../../../services/pluginsManager/pluginsEventBus');

jest.mock('./findClosestReactionPoint', () => ({
  __esModule: true,
  ...jest.requireActual('./findClosestReactionPoint'),
}));

const findClosestReactionPointSpy = jest.spyOn(
  findClosestReactionPoint,
  'findClosestReactionPoint',
);

const SEARCH_CONFIG_MOCK = {
  point: {
    x: 200,
    y: 3012,
  },
  maxZoom: 9,
  zoom: 3,
  isResultDrawerOpen: false,
};

describe('handleReactionResults - util', () => {
  const searchDistance = '10';

  beforeEach(() => {
    jest.clearAllMocks();
  });

  describe('actions', () => {
    const { store } = getReduxStoreWithActionsListener({
      ...INITIAL_STORE_STATE_MOCK,
    });
    const { dispatch } = store;

    mockedAxiosNewClient
      .onGet(
        apiPath.getBioEntityContentsStringWithQuery({
          searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
          isPerfectMatch: true,
        }),
      )
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    mockedAxiosOldClient
      .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
      .reply(HttpStatusCode.Ok, [
        {
          ...reactionsFixture[0],
          reactants: [],
          products: [],
          modifiers: [
            {
              ...reactionsFixture[0].modifiers[0],
              aliasId: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            },
          ],
        },
      ]);

    beforeEach(async () => {
      await handleReactionResults(
        dispatch,
        ELEMENT_SEARCH_RESULT_MOCK_REACTION,
      )(ELEMENT_SEARCH_RESULT_MOCK_REACTION);
    });

    it('should run getReactionsByIds as first action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[0].type).toEqual('reactions/getByIds/pending');
      expect(actions[1].type).toEqual('reactions/getByIds/fulfilled');
    });

    it('should run openReactionDrawerById to empty array as third action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[2].type).toEqual('drawer/openReactionDrawerById');
      expect(actions[2].payload).toEqual(reactionsFixture[FIRST_ARRAY_ELEMENT].id);
    });

    it('should run select tab as fourth action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[3].type).toEqual('drawer/selectTab');
    });

    it('should run getMultiBioEntity to empty array as fifth action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[4].type).toEqual('project/getMultiBioEntity/pending');
    });

    it('should run getBioEntityContents as sixth action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[5].type).toEqual('project/getBioEntityContents/pending');
    });

    it('should run getBioEntityContents fullfilled as seventh action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[6].type).toEqual('project/getBioEntityContents/fulfilled');
    });

    it('should run addNumbersToEntityNumberData as eighth action', () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[7].type).toEqual('entityNumber/addNumbersToEntityNumberData');
    });
  });
  describe('when search config provided but search distance is not provided', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;

    it('should not find closest reaction', async () => {
      await handleReactionResults(
        dispatch,
        ELEMENT_SEARCH_RESULT_MOCK_REACTION,
        SEARCH_CONFIG_MOCK,
      )(ELEMENT_SEARCH_RESULT_MOCK_REACTION);

      expect(findClosestReactionPointSpy).not.toHaveBeenCalled();
    });
  });

  describe('when search config provided and matching reaction not found', () => {
    mockedAxiosNewClient
      .onGet(
        apiPath.getBioEntityContentsStringWithQuery({
          searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
          isPerfectMatch: true,
        }),
      )
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    mockedAxiosOldClient
      .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
      .reply(HttpStatusCode.Ok, [
        {
          ...reactionsFixture[0],
          reactants: [],
          products: [],
          modifiers: [
            {
              ...reactionsFixture[0].modifiers[0],
              aliasId: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            },
          ],
        },
      ]);

    const invalidPoint = {
      x: 991,
      y: 612,
    };
    it('should close drawer and reset data if result drawer open', async () => {
      const { store } = getReduxStoreWithActionsListener();

      await handleReactionResults(store.dispatch, ELEMENT_SEARCH_RESULT_MOCK_REACTION, {
        ...SEARCH_CONFIG_MOCK,
        searchDistance,
        point: invalidPoint,
        isResultDrawerOpen: true,
      })(ELEMENT_SEARCH_RESULT_MOCK_REACTION);

      const actions = store.getActions();
      const acionTypes = actions.map(action => action.type);

      expect(acionTypes).toStrictEqual([
        'reactions/getByIds/pending',
        'reactions/getByIds/fulfilled',
        'drawer/closeDrawer',
        'reactions/resetReactionsData',
        'bioEntityContents/clearBioEntitiesData',
      ]);
    });

    it('should only reset data if result drawer is closed', async () => {
      const { store } = getReduxStoreWithActionsListener();

      const dispatchSpy = jest.spyOn(store, 'dispatch');

      await handleReactionResults(store.dispatch, ELEMENT_SEARCH_RESULT_MOCK_REACTION, {
        ...SEARCH_CONFIG_MOCK,
        searchDistance,
        point: invalidPoint,
        isResultDrawerOpen: false,
      })(ELEMENT_SEARCH_RESULT_MOCK_REACTION);

      expect(dispatchSpy).toHaveBeenCalledWith({
        payload: undefined,
        type: 'reactions/resetReactionsData',
      });

      expect(dispatchSpy).toHaveBeenCalledWith({
        payload: undefined,
        type: 'bioEntityContents/clearBioEntitiesData',
      });
    });
  });
  describe('when search config provided and matching reaction found', () => {
    const reaction = {
      ...reactionsFixture[0],
      products: [],
      reactants: [],
      modifiers: [],
      lines: [{ start: { x: 0, y: 0 }, end: { x: 3, y: 4 }, type: 'START' }],
    };

    const point = { x: 1, y: 1 };
    const maxZoom = 10;
    const zoom = 5;

    it('should open reaction drawer and fetch bio entities', async () => {
      const { store } = getReduxStoreWithActionsListener();
      mockedAxiosNewClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
            isPerfectMatch: true,
          }),
        )
        .reply(HttpStatusCode.Ok, []);

      mockedAxiosOldClient
        .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
        .reply(HttpStatusCode.Ok, [reaction]);

      await handleReactionResults(store.dispatch, ELEMENT_SEARCH_RESULT_MOCK_REACTION, {
        searchDistance,
        maxZoom,
        zoom,
        point,
        isResultDrawerOpen: false,
      })(ELEMENT_SEARCH_RESULT_MOCK_REACTION);

      const actions = store.getActions();
      const acionTypes = actions.map(action => action.type);

      expect(acionTypes).toStrictEqual([
        'reactions/getByIds/pending',
        'reactions/getByIds/fulfilled',
        'drawer/openReactionDrawerById',
        'drawer/selectTab',
        'project/getMultiBioEntity/pending',
        'entityNumber/addNumbersToEntityNumberData',
        'project/getMultiBioEntity/fulfilled',
      ]);
    });
  });
  describe('when matching reaction found', () => {
    const reaction = {
      ...reactionsFixture[0],
      products: [],
      reactants: [],
      modifiers: [],
      lines: [{ start: { x: 0, y: 0 }, end: { x: 3, y: 4 }, type: 'START' }],
    };

    const point = { x: 1, y: 1 };
    const maxZoom = 10;
    const zoom = 5;

    it('should dispatch onSearch event with reaction data', async () => {
      const { store } = getReduxStoreWithActionsListener();
      mockedAxiosNewClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
            isPerfectMatch: true,
          }),
        )
        .reply(HttpStatusCode.Ok, []);

      mockedAxiosOldClient
        .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
        .reply(HttpStatusCode.Ok, [reaction]);

      await handleReactionResults(store.dispatch, ELEMENT_SEARCH_RESULT_MOCK_REACTION, {
        searchDistance,
        maxZoom,
        zoom,
        point,
        isResultDrawerOpen: false,
      })(ELEMENT_SEARCH_RESULT_MOCK_REACTION);

      expect(PluginsEventBus.dispatchEvent).toHaveBeenCalledWith('onSearch', {
        results: [[reaction]],
        searchValues: [ELEMENT_SEARCH_RESULT_MOCK_REACTION],
        type: 'reaction',
      });
    });
  });
});
