import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import contentsReducer from './bioEntity.slice';
import { getBioEntity, getMultiBioEntity } from './bioEntity.thunks';
import { BioEntityContentsState } from './bioEntity.types';

const mockedAxiosClient = mockNetworkNewAPIResponse();
const SEARCH_QUERY = 'park7';

describe('bioEntityContents thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioEntityContentsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('bioEntityContents', contentsReducer);
  });
  describe('getBioEntityContents', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: SEARCH_QUERY,
            isPerfectMatch: false,
          }),
        )
        .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

      const { payload } = await store.dispatch(
        getBioEntity({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      );
      expect(payload).toEqual(bioEntityResponseFixture.content);
    });
    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: SEARCH_QUERY,
            isPerfectMatch: false,
          }),
        )
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(
        getBioEntity({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      );
      expect(payload).toEqual(undefined);
    });
    it('should handle error message when getBioEntityContents failed', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: SEARCH_QUERY,
            isPerfectMatch: false,
          }),
        )
        .reply(HttpStatusCode.NotFound, null);

      const { payload } = await store.dispatch(
        getBioEntity({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      );
      expect(payload).toEqual(
        "Failed to fetch bio entity: The page you're looking for doesn't exist. Please verify the URL and try again.",
      );
    });
  });
  describe('getMultiBioEntity', () => {
    it('should return transformed bioEntityContent array', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: SEARCH_QUERY,
            isPerfectMatch: false,
          }),
        )
        .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

      const data = await store
        .dispatch(
          getMultiBioEntity({
            searchQueries: [SEARCH_QUERY],
            isPerfectMatch: false,
          }),
        )
        .unwrap();

      expect(data).toEqual(bioEntityResponseFixture.content);
    });
    it('should combine all returned bioEntityContent arrays and return array with all provided bioEntityContent elements', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getBioEntityContentsStringWithQuery({
            searchQuery: SEARCH_QUERY,
            isPerfectMatch: false,
          }),
        )
        .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

      const data = await store
        .dispatch(
          getMultiBioEntity({
            searchQueries: [SEARCH_QUERY, SEARCH_QUERY],
            isPerfectMatch: false,
          }),
        )
        .unwrap();

      expect(data).toEqual([
        ...bioEntityResponseFixture.content,
        ...bioEntityResponseFixture.content,
      ]);
    });
  });
});
